function solar_transform_zfs(elem, cmd, rc, out) {
	var tbl = new TableTransformer()
	
	// http://docs.oracle.com/cd/E19963-01/html/821-1448/gazss.html#gcfgr
	tbl.header("aclinherit", "Controls how ACL entries are inherited when files and directories are created. The values arediscard,noallow,secure, andpassthrough.")
	tbl.header("atime", "Controls whether the access time for files is updated when they are read. Turning this property off avoids producing write traffic when reading files and can result in significant performance gains, though it might confuse mailers and similar utilities.")
	tbl.header("available", "Read-only property that identifies the amount of disk space available to a dataset and all its children, assuming no other activity in the pool. Because disk space is shared within a pool, available space can be limited by various factors including physical pool size, quotas, reservations, and other datasets within the pool.")
	tbl.header("avail", "Read-only property that identifies the amount of disk space available to a dataset and all its children, assuming no other activity in the pool. Because disk space is shared within a pool, available space can be limited by various factors including physical pool size, quotas, reservations, and other datasets within the pool.")
	tbl.header("canmount", "Controls whether a file system can be mounted with thezfsmountcommand. This property can be set on any file system, and the property itself is not inheritable. However, when this property is set tooff, a mount point can be inherited to descendent file systems, but the file system itself is never mounted.")
	tbl.header("casesensitivity", "This property indicates whether the file name matching algorithm used by the file system should becasesensitive,caseinsensitive, or allow a combination of both styles of matching (mixed). Traditionally, UNIX and POSIX file systems have case-sensitive file names.")
	tbl.header("checksum", "Controls the checksum used to verify data integrity. The default value ison, which automatically selects an appropriate algorithm, currentlyfletcher4. The values areon, off, fletcher2,fletcher4,sha256, andsha256+mac. A value ofoffdisables integrity checking on user data. A value ofoffis not recommended.")
	tbl.header("compression", "Enables or disables compression for a dataset. The values areon,off,lzjb,gzip, andgzip-N. Currently, setting this property tolzjb,gzip, orgzip-Nhas the same effect as setting this property toon. Enabling compression on a file system with existing data only compresses new data. Existing data remains uncompressed.")
	tbl.header("compress", "Enables or disables compression for a dataset. The values areon,off,lzjb,gzip, andgzip-N. Currently, setting this property tolzjb,gzip, orgzip-Nhas the same effect as setting this property toon. Enabling compression on a file system with existing data only compresses new data. Existing data remains uncompressed.")
	tbl.header("compressratio", "Read-only property that identifies the compression ratio achieved for a dataset, expressed as a multiplier. Compression can be enabled by thezfs set compression=ondatasetcommand.")
	tbl.header("copies", "Sets the number of copies of user data per file system. Available values are1,2, or3. These copies are in addition to any pool-level redundancy. Disk space used by multiple copies of user data is charged to the corresponding file and dataset, and counts against quotas and reservations. In addition, theusedproperty is updated when multiple copies are enabled. Consider setting this property when the file system is created because changing this property on an existing file system only affects newly written data.")
	tbl.header("creation", "Read-only property that identifies the date and time that a dataset was created.")
	tbl.header("dedup", "Controls the ability to remove duplicate data in a ZFS file system. Possible values areon,off,verify, andsha256[,verify].. The default checksum for deduplication issha256.")
	tbl.header("devices", "Controls whether device files in a file system can be opened.")
	tbl.header("encryption", "Controls whether a dataset is encrypted. An encrypted dataset means that data is encoded and a key is needed by the dataset owner to access the data.")
	tbl.header("exec", "Controls whether programs in a file system are allowed to be executed. Also, when set tooff,mmap(2)calls withPROT_EXECare disallowed.")
	tbl.header("keysource", "Identifies the format and location of the key that wraps the dataset keys. The valid property values areraw,hex,passphrase,prompt, orfile. The key must be present when the dataset is created, mounted, or loaded by using thezfs key-lcommand. If encryption is enabled for a new dataset, the defaultkeysourceispassphrase,prompt.")
	tbl.header("keystatus", "Read-only property that identifies the dataset's encryption key status. The availability of a dataset's key is indicated byavailableorunavailable. For datasets that do not have encryption enabled,noneis displayed.")
	tbl.header("logbias", "Controls how ZFS optimizes synchronous requests for this dataset. Iflogbiasis set tolatency, ZFS uses the pool's separate log devices, if any, to handle the requests at low latency. Iflogbiasis set tothroughput, ZFS does not use the pool's separate log devices. Instead, ZFS optimizes synchronous operations for global pool throughput and efficient use of resources. The default value islatency.")
	tbl.header("mlslabel", "Provides a sensitivity label that determines if a dataset can be mounted in a Trusted Extensions zone. If the labeled dataset matches the labeled zone, the dataset can be mounted and accessed from the labeled zone. The default value isnone. This property can only be modified when Trusted Extensions is enabled and only with the appropriate privilege.")
	tbl.header("mounted", "Read-only property that indicates whether a file system, clone, or snapshot is currently mounted. This property does not apply to volumes. The value can be eitheryesorno.")
	tbl.header("mountpoint", "Controls the mount point used for this file system. When themountpointproperty is changed for a file system, the file system and any descendents that inherit the mount point are unmounted. If the new value islegacy, then they remain unmounted. Otherwise, they are automatically remounted in the new location if the property was previouslylegacyornone, or if they were mounted before the property was changed. In addition, any shared file systems are unshared and shared in the new location.")
	tbl.header("primarycache", "Controls what is cached in the primary cache (ARC). Possible values areall,none, andmetadata. If set toall, both user data and metadata are cached. If set tonone, neither user data nor metadata is cached. If set tometadata, only metadata is cached.")
	tbl.header("nbmand", "Controls whether the file system should be mounted withnbmand(Non-blocking mandatory) locks. This property is for SMB clients only. Changes to this property only take effect when the file system is unmounted and remounted.")
	tbl.header("normalization", "This property indicates whether a file system should perform a unicode normalization of file names whenever two file names are compared, and which normalization algorithm should be used. File names are always stored unmodified, names are normalized as part of any comparison process. If this property is set to a legal value other thannone, and theutf8onlyproperty was left unspecified, theutf8onlyproperty is automatically set toon. The default value of thenormalizationproperty isnone. This property cannot be changed after the file system is created.")
	tbl.header("origin", "Read-only property for cloned file systems or volumes that identifies the snapshot from which the clone was created. The origin cannot be destroyed (even with the-ror-foption) as long as a clone exists.")
	tbl.header("quota", "Limits the amount of disk space a dataset and its descendents can consume. This property enforces a hard limit on the amount of disk space used, including all space consumed by descendents, such as file systems and snapshots. Setting a quota on a descendent of a dataset that already has a quota does not override the ancestor's quota, but rather imposes an additional limit. Quotas cannot be set on volumes, as thevolsizeproperty acts as an implicit quota.")
	tbl.header("rekeydate", "Read-only property that indicates the date of the last data encryption key change from azfs key-Korzfs clone-Koperation on this dataset. If norekeyoperation has been performed, the value of this property is the same as thecreationdate.")
	tbl.header("readonly", "Controls whether a dataset can be modified. When set toon, no modifications can be made.")
	tbl.header("rdonly", "Controls whether a dataset can be modified. When set toon, no modifications can be made.")
	tbl.header("recordsize", "Specifies a suggested block size for files in a file system.")
	tbl.header("recsize", "Specifies a suggested block size for files in a file system.")
	tbl.header("referenced", "Read-only property that identifies the amount of data accessible by a dataset, which might or might not be shared with other datasets in the pool.")
	tbl.header("refer", "Read-only property that identifies the amount of data accessible by a dataset, which might or might not be shared with other datasets in the pool.")
	tbl.header("refquota", "Sets the amount of disk space that a dataset can consume. This property enforces a hard limit on the amount of space used. This hard limit does not include disk space used by descendents, such as snapshots and clones.")
	tbl.header("refreservation", "Sets the minimum amount of disk space that is guaranteed to a dataset, not including descendents, such as snapshots and clones. When the amount of disk space used is below this value, the dataset is treated as if it were taking up the amount of space specified byrefreservation. Therefreservationreservation is accounted for in the parent dataset's disk space used, and counts against the parent dataset's quotas and reservations.")
	tbl.header("refreserv", "Sets the minimum amount of disk space that is guaranteed to a dataset, not including descendents, such as snapshots and clones. When the amount of disk space used is below this value, the dataset is treated as if it were taking up the amount of space specified byrefreservation. Therefreservationreservation is accounted for in the parent dataset's disk space used, and counts against the parent dataset's quotas and reservations.")
	tbl.header("reservation", "Sets the minimum amount of disk space guaranteed to a dataset and its descendents. When the amount of disk space used is below this value, the dataset is treated as if it were using the amount of space specified by its reservation. Reservations are accounted for in the parent dataset's disk space used, and count against the parent dataset's quotas and reservations.")
	tbl.header("reserv", "Sets the minimum amount of disk space guaranteed to a dataset and its descendents. When the amount of disk space used is below this value, the dataset is treated as if it were using the amount of space specified by its reservation. Reservations are accounted for in the parent dataset's disk space used, and count against the parent dataset's quotas and reservations.")
	tbl.header("rstchown", "Indicates whether the file system owner can grant file ownership changes. The default is to restrictchownoperations. Whenrstchownis set to off, the user has thePRIV_FILE_CHOWN_SELFprivilege forchownoperations.")
	tbl.header("secondarycache", "Controls what is cached in the secondary cache (L2ARC). Possible values areall,none, andmetadata. If set toall, both user data and metadata are cached. If set tonone, neither user data nor metadata is cached. If set tometadata, only metadata is cached.")
	tbl.header("setuid", "Controls whether thesetuidbit is honored in a file system.")
	tbl.header("sharenfs", "Controls whether a file system is available over NFS and what options are used. If set toon, thezfs sharecommand is invoked with no options. Otherwise, thezfs sharecommand is invoked with options equivalent to the contents of this property. If set tooff, the file system is managed by using the legacyshareandunsharecommands and thedfstabfile.")
	tbl.header("sharesmb", "Controls whether the file system is shared by using the Oracle Solaris SMB service, and what options are to be used. A file system with thesharesmbproperty set tooffis managed through traditional tools, such as thesharemgrcommand. Otherwise, the file system is automatically shared and unshared by using thezfs shareandzfs unsharecommands.")
	tbl.header("snapdir", "Controls whether the.zfsdirectory is hidden or visible in the root of the file system.")
	tbl.header("sync", "Determines the synchronous behavior of a file system's transactions. Possible values are:standard, the default value, which means synchronous file system transactions, such asfsync,O_DSYNC,O_SYNC, and so on, are written to the intent log;always, ensures thateveryfile system transaction is written and flushed to stable storage by a returning system call. This value has a significant performance penalty;disabled, means that synchronous requests are disabled. File system transactions are only committed to stable storage on the next transaction group commit, which might be after many seconds. This value gives the best performance, with no risk of corrupting the pool. However, this value is very dangerous because ZFS is ignoring the synchronous transaction demands of applications, such as databases or NFS operations. Setting this value on the currently active root or/varfile system might result in out-of-spec behavior, application data loss, or increased vulnerability to replay attacks. You should only use this value if you fully understand all the associated risks.")
	tbl.header("type", "Read-only property that identifies the dataset type asfilesystem(file system or clone),volume, orsnapshot.")
	tbl.header("used", "Read-only property that identifies the amount of disk space consumed by a dataset and all its descendents.")
	tbl.header("usedbychildren", "Read-only property that identifies the amount of disk space that is used by children of this dataset, which would be freed if all the dataset's children were destroyed. The property abbreviation isusedchild.")
	tbl.header("usedchild", "Read-only property that identifies the amount of disk space that is used by children of this dataset, which would be freed if all the dataset's children were destroyed. The property abbreviation isusedchild.")
	tbl.header("usedbydataset", "Read-only property that identifies the amount of disk space that is used by a dataset itself, which would be freed if the dataset was destroyed, after first destroying any snapshots and removing anyrefreservationreservations. The property abbreviation isusedds.")
	tbl.header("usedbyrefreservation", "Read-only property that identifies the amount of disk space that is used by arefreservationset on a dataset, which would be freed if therefreservationwas removed. The property abbreviation isusedrefreserv.")
	tbl.header("usedrefreserv", "Read-only property that identifies the amount of disk space that is used by arefreservationset on a dataset, which would be freed if therefreservationwas removed. The property abbreviation isusedrefreserv.")
	tbl.header("usedbysnapshots", "Read-only property that identifies the amount of disk space that is consumed by snapshots of a dataset. In particular, it is the amount of disk space that would be freed if all of this dataset's snapshots were destroyed. Note that this value is not simply the sum of the snapshots'usedproperties, because space can be shared by multiple snapshots. The property abbreviation isusedsnap.")
	tbl.header("usedsnap", "Read-only property that identifies the amount of disk space that is consumed by snapshots of a dataset. In particular, it is the amount of disk space that would be freed if all of this dataset's snapshots were destroyed. Note that this value is not simply the sum of the snapshots'usedproperties, because space can be shared by multiple snapshots. The property abbreviation isusedsnap.")
	tbl.header("version", "Identifies the on-disk version of a file system, which is independent of the pool version. This property can only be set to a later version that is available from the supported software release.")
	tbl.header("utf8only", "This property indicates whether a file system should reject file names that include characters that are not present in the UTF-8 character code set. If this property is explicitly set tooff, thenormalizationproperty must either not be explicitly set or be set tonone. The default value for theutf8onlyproperty isoff. This property cannot be changed after the file system is created.")
	tbl.header("volsize", "For volumes, specifies the logical size of the volume.")
	tbl.header("volblocksize", "For volumes, specifies the block size of the volume. The block size cannot be changed after the volume has been written, so set the block size at volume creation time. The default block size for volumes is 8 KB. Any power of 2 from 512 bytes to 128 KB is valid.")
	tbl.header("volblock", "For volumes, specifies the block size of the volume. The block size cannot be changed after the volume has been written, so set the block size at volume creation time. The default block size for volumes is 8 KB. Any power of 2 from 512 bytes to 128 KB is valid.")
	tbl.header("vscan", "Controls whether regular files should be scanned for viruses when a file is opened and closed. In addition to enabling this property, a virus scanning service must also be enabled for virus scanning to occur if you have third-party virus scanning software. The default value isoff.")
	tbl.header("zoned", "Indicates whether a dataset has been added to a non-global zone. If this property is set, then the mount point is not honored in the global zone, and ZFS cannot mount such a file system when requested. When a zone is first installed, this property is set for any added file systems.")
	tbl.header("xattr", "Indicates whether extended attributes are enabled (on) or disabled (off) for this file system.")

	tbl.create(out, elem)
	
	return "Table"
}